<?php

namespace Drupal\migrate_media_handler;

use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Entity\EntityTypeManager;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\migrate\Row;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * MediaMaker takes file paths and makes file & media entities if possible.
 *
 * Requires the presence of a text field on all destination media entities
 * named `field_original_ref`. This allows for lookup by file hash to avoid
 * duplication. This field can be deleted after migration is complete.
 */
class MediaMaker implements ContainerInjectionInterface {

  /**
   * Entity Type Manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManager
   */
  protected $entityManager;

  /**
   * ConfigFactory service.
   *
   * @var \Drupal\Core\Config\ConfigFactory
   */
  protected $configFactory;

  /**
   * Config variable.
   *
   * @var \Drupal\Core\Config\Config
   */
  protected $config;

  /**
   * {@inheritdoc}
   */
  public function __construct(EntityTypeManager $entity_manager, ConfigFactory $config_factory) {
    $this->entityManager = $entity_manager;
    $this->configFactory = $config_factory;

    // Pull media setting from config. Can be overridden if necessary.
    $this->config = $this->configFactory->get('migrate_media_handler.settings');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('config.factory')
    );
  }

  /**
   * Find existing media by file hash.
   *
   * @param string $hash
   *   File hash to lookup in field_original_ref.
   *
   * @return bool|\Drupal\media\Entity\Media
   *   Return media entity or FALSE
   */
  public function findExistingMediaByHash(string $hash) {
    $media = FALSE;
    // Lookup the media entity by hash of file data in field_original_ref.
    $query = $this->entityManager->getStorage('media')->getQuery()
      ->condition('status', 1)
      ->condition('field_original_ref', $hash, '=');
    $mids = $query->execute();
    // If found, `mid` is the reference for the content.
    if (!empty($mids)) {
      $mid = array_pop($mids);
      // This check is to make sure that the media /actually/ exists.
      $media = $this->entityManager->getStorage('media')->load($mid);
    }
    return $media;
  }

  /**
   * Find existing media by file path, using the hash.
   *
   * @param string $path
   *   Path from which to generate hash for comparison.
   *
   * @return bool|\Drupal\media\Entity\Media
   *   Return media entity or FALSE
   */
  public function findExistingMediaByPath(string $path) {
    // Hash file, then find by stored hash.
    $hash = sha1_file($path);
    return $this->findExistingMediaByHash($hash);
  }

  /**
   * Make a file entity out of a file path. Move file to current file directory.
   *
   * @param string $path
   *   File path of existing legacy file.
   *
   * @return bool|\Drupal\file\Entity\File
   *   File entity or FALSE
   */
  public function makeFileEntity(string $path) {
    $destination = str_ireplace($this->config->get('file_source'), $this->config->get('file_dest'), $path);
    $filename = pathinfo($path, PATHINFO_BASENAME);

    if (file_exists($path)) {
      // Create file entity.
      $file = $this->entityManager->getStorage('file')->create();
      $file->setFileUri($destination);
      $file->setFilename($filename);
      $file->setMimeType(mime_content_type($path));
      $file->setSize(filesize($path));
      $file->setOwnerId($this->config->get('file_owner'));
      $file->setPermanent();
      $file->save();

      // Copy file to permanent destination.
      $dir = dirname($destination);
      if (!file_exists($dir)) {
        mkdir($dir, 0770, TRUE);
      }
      file_put_contents($destination, file_get_contents($path));
      $file->save();
      return $file;
    }
    return FALSE;
  }

  /**
   * Make an image media entity out of a file. Add alt, title if avail.
   *
   * @param int $file_id
   *   ID of existing file entity.
   * @param Drupal\migrate\Row $row
   *   Migration row, used for getting title data.
   * @param array $configuration
   *   Config of process plugin.
   *
   * @return bool|\Drupal\media\Entity\Media
   *   Return media entity or FALSE
   */
  public function makeImageEntity(int $file_id, Row $row, array $configuration) {
    $media = FALSE;
    // Load file entity.
    $file = $this->entityManager->getStorage('file')->load($file_id);
    // If that's successful, carry on.
    if ($file) {
      // Hash file for overlap lookup.
      $hash = sha1_file($file->getFileUri());

      // Lookup media entity by file hash.
      $media = $this->findExistingMediaByHash($hash);

      // If it wasn't found, make it!
      if (!$media) {
        $field_source = $row->getSourceProperty($configuration['source_field']);
        // Create media entity with saved file.
        // Please note accessibility concerns around empty alt & title.
        $media = $this->entityManager->getStorage('media')->create([
          'bundle' => 'image',
          'field_original_ref' => $hash,
          $this->config->get('image_field_name') => [
            'target_id' => $file_id,
            'alt' => $field_source[0]['alt'],
            'title' => $field_source[0]['title'],
          ],
          'langcode' => 'en',
        ]);
        $owner = $file->getOwnerId();
        $filename = $file->getFilename();
        $media->setOwnerId($owner);
        $media->setName($filename);
        $media->save();
      }
    }
    return $media;
  }

  /**
   * Make a document media entity out of a file.
   *
   * @param string $file_id
   *   ID of existing file entity.
   * @param string $hash
   *   File hash of existing legacy file.
   * @param string $display
   *   Display value (0 or 1).
   * @param string $description
   *   File description.
   *
   * @return bool|\Drupal\media\Entity\Media
   *   Return media entity or FALSE
   */
  public function makeDocumentEntity($file_id, $hash = '', $display = '', $description = '') {
    $media = FALSE;
    // Load file entity.
    $file = $this->entityManager->getStorage('file')->load($file_id);
    // If that's successful, carry on.
    if ($file) {
      // If no file hash was specified, hash it.
      if (!$hash) {
        $hash = sha1_file($file->getFileUri());
      }
      // Lookup media entity by file hash.
      $media = $this->findExistingMediaByHash($hash);

      // If it wasn't found, make it!
      if (!$media) {
        // Create media entity with saved file.
        $media = $this->entityManager->getStorage('media')->create([
          'bundle' => 'document',
          'field_original_ref' => $hash,
          $this->config->get('document_field_name') => [
            'target_id' => $file_id,
            'display' => $display,
            'description' => $description,
          ],
          'langcode' => 'en',
        ]);
        // Get additional media properties.
        $owner = $file->getOwnerId();
        $filename = $file->getFilename();
        // Set additional media properties.
        $media->setOwnerId($owner);
        $media->setName($filename);

        $media->save();

      }
    }
    return $media;
  }

  /**
   * Make a video media entity out of a url.
   *
   * @param array $field_data
   *   Array of data from the D7 video embed field.
   * @param Drupal\migrate\Row $row
   *   Migration row, used for getting title data.
   *
   * @return bool|\Drupal\media\Entity\Media
   *   Return media entity or FALSE
   */
  public function makeVideoLinkEntity(array $field_data, Row $row) {
    $media = FALSE;
    if (!empty($field_data['video_url'])) {
      $hash = sha1($field_data['video_url']);
      // Lookup media entity by hash.
      $media = $this->findExistingMediaByHash($hash);
      // If it wasn't found, make it!
      if (!$media) {
        $title = $row->getSourceProperty('title');
        $langcode = $row->getSourceProperty('language');
        // Create media entity with url.
        $media = $this->entityManager->getStorage('media')->create([
          'name' => $title,
          'bundle' => 'video',
          'field_original_ref' => $hash,
          $this->config->get('video_field_name') => $field_data['video_url'],
          'langcode' => $langcode,
        ]);

        $media->save();

      }
    }
    return $media;
  }

  /**
   * Make an audio media entity out of a file.
   *
   * @param string $file_id
   *   ID of existing file entity.
   * @param string $hash
   *   File hash of existing legacy file.
   * @param string $display
   *   Display value (0 or 1).
   * @param string $description
   *   File description.
   *
   * @return bool|\Drupal\media\Entity\Media
   *   Return media entity or FALSE
   */
  public function makeAudioEntity($file_id, $hash = '', $display = '', $description = '') {
    $media = FALSE;
    // Load file entity.
    $file = $this->entityManager->getStorage('file')->load($file_id);
    // If that's successful, carry on.
    if ($file) {
      // If no file hash was specified, hash it.
      if (!$hash) {
        $hash = sha1_file($file->getFileUri());
      }
      // Lookup media entity by file hash.
      $media = $this->findExistingMediaByHash($hash);

      // If it wasn't found, make it!
      if (!$media) {
        $file_field = $this->config->get('audio_field_name');

        // Create media entity with saved file.
        $media = $this->entityManager->getStorage('media')->create([
          'bundle' => 'audio',
          'field_original_ref' => $hash,
          $file_field => [
            'target_id' => $file_id,
            'display' => $display,
            'description' => $description,
          ],
          'langcode' => 'en',
        ]);
        // Get additional media properties.
        $owner = $file->getOwnerId();
        $filename = $file->getFilename();
        // Set additional media properties.
        $media->setOwnerId($owner);
        $media->setName($filename);

        $media->save();

      }
    }
    return $media;
  }

  /**
   * Utility function to get source file repository filepath.
   *
   * @param string $path
   *   Path from which to generate hash for comparison.
   * @param bool $find_in_store
   *   Replace "public://" with file_source variable.
   */
  public function getSourceFilePath($path = '', $find_in_store = FALSE) {
    // If link is full-path (contains http:// or https://).
    if (preg_match('/https?:\/\//', $path) !== 0) {
      // If link is full-path to existing site.
      if (preg_match($this->config->get('site_uri'), $path) !== 0) {
        // Swap out full path for public://, and remove sites/default/files.
        $file_path = preg_replace($this->config->get('site_uri'), $this->config->get('file_source'), $path);
        $file_path = str_ireplace("/sites/default/files", "", $file_path);
      }
      // Otherwise, it's an external link.
      else {
        $file_path = $path;
      }
    }
    // Find in File Store.
    if ($find_in_store && stripos($path, 'public://') === 0 && stripos($path, $this->config->get('file_source')) === FALSE) {
      $file_path = str_ireplace("public://", $this->config->get('file_source') . "/", $path);
    }
    // Otherwise, it's just a relative link.
    else {
      $file_path = $path;
    }

    return $file_path;
  }

}
